/* $Id: 1$ */
/** @acpi.c
  *
  *  API interface for ACPI
  *
  * netlabs.org confidential
  *
  * Copyright (c) 2005 netlabs.org
  *
  * Author: Pavel Shtemenko <pasha@paco.odessa.ua>
  *
  * All Rights Reserved
  *
  */
#ifndef __ACPIAPI_H__
#define __ACPIAPI_H__

#pragma pack(1)

#define DRV_NAME "ACPICA$"
typedef struct _VersionAcpi_
               {
               ULONG  Major;
               ULONG  Minor;
               } ACPI_VERSION;
typedef struct _AcpiApiHandle_
               {
               HFILE           AcpiDrv;                 // Handle to ACPICA driver
               ACPI_VERSION    PSD;                     // Version PSD
               ACPI_VERSION    Driver;                  // Version ACPICA driver
               ACPI_VERSION    DLL;                     // Version acpi32.dll
               ULONG           StartAddrPSD;            // Start address PSD (for testcase)
               ULONG           AddrCommApp;             // Address DosCommApp from PSD (which not write IBM)
               ULONG           StartAddrDriver;         // Start address ACPICA (for testcase)
               ULONG           AddrFindPSD;             // Address function for find PSD (find CommApp)
               ULONG           IRQNumber;               // Number use IRQ
               void            *Internal;               // For internal DLL use
               } ACPI_API_HANDLE, *PACPI_API_HANDLE;

typedef struct _LAPICError_
              {

               UINT32 SendChecksum;
               UINT32 ReceiveChecksum;
               UINT32 SendAccept;
               UINT32 ReceiveAccept;
               UINT32 SendIllegal;
               UINT32 ReceiveIllegal;
               UINT32 IllegalRegAddress;
              } LAPICError, *PLAPICError;

#define MAX_CSTATE  3
typedef struct _Processor_
               {
               ACPI_IO_ADDRESS      PblkAddress;                            // Address P_BLK
               UINT32               PblkLength;                             // Length  P_BLK
               UINT8                Width;                                  // Width of trottling
               UINT8                Offset;                                 // ofsset from begin
               UINT8                Lint;                                   // Field from MADT
               UINT8                LocalApicId;                            // Field from MADT
               UINT32               ProcessorId;                            // Field from MADT
               LAPICError           APIC;                                   // Local APIC error counter
               ACPI_HANDLE          Handle;                                 // Handle to ACPI tree
               ACPI_GENERIC_ADDRESS PerfControl;                            // Perfomance control
               ACPI_GENERIC_ADDRESS PerfStatus;                             // Perfomance control
               UINT32               CxLatency[MAX_CSTATE];                  // C state latency
               ACPI_GENERIC_ADDRESS CxState[MAX_CSTATE];                    // C state control register
               } PROCESSOR, *PPROCESSOR;
typedef struct _IRQ_
               {
               UINT32     Triggering;                         // Edge/Level
               UINT32     Polarity;                           // High/Low
               UINT32     Sharable;                           // Yes/No
               UINT32     Vector;                             // Vector for IDT
               ULONG      *IOApicAddress;                     // Address IO APIC for this IRQ
               UINT32     IRQNumber;                          // IRQ number for IO APIC (not EQ to IRQ on bus)
               UINT32     SetIRQLow;
               UINT32     SetIRQHigh;
               UINT32     (*Handler)(void);
               UINT64     Count;                              // Called count
               } AcpiIRQ, *PAcpiIRQ;

typedef struct _IRQBus_
               {
               char            *Name;                    // Name in ACPI tree
               UINT32          BusNumber;                // Bus number for all devices in this name
               ACPI_OBJECT     *Pointer;                 // Pointer to routing table for this bus
               } IRQBus, *PIRQBus;

typedef struct _IOAPIC_
              {
              UINT32     PhysAddr;
              UINT32     IOApicId;
              UINT32     BaseIRQ;
              } IOAPIC, *PIOAPIC;

typedef struct _ACPI_STATISTICS_
              {
              UINT32    Command;
              UINT32    Number;
              UINT32    Status;
              PAcpiIRQ  Data;
              }  ACPI_STATISTICS, *PACPI_STATISTICS;

typedef struct _ACPI_Notify_
               {
               UINT32        Number;                                 // Message Number
               ACPI_HANDLE   Handle;                                 // Handle to ACPI device
               } ACPINotify, *PACPINotify;

#define ACPI_STATISTICS_GETSIZEIRQ    1
#define ACPI_STATISTICS_GETIRQ        2
#define ACPI_STATISTICS_CLEARIRQ      3
#define ACPI_STATISTICS_GETSIZECPU    4
#define ACPI_STATISTICS_GETCPU        5


#ifdef __cplusplus
extern "C" {
#endif

APIRET APIENTRY AcpiStartApi(ACPI_API_HANDLE *);
APIRET APIENTRY AcpiEndApi(ACPI_API_HANDLE *);
APIRET APIENTRY AcpiWaitPressButton(ACPI_API_HANDLE *,ULONG,ULONG,ULONG *);
APIRET APIENTRY AcpiGoToSleep(ACPI_API_HANDLE *,UINT8);
APIRET APIENTRY AcpiGetUpTime(ACPI_API_HANDLE *,UINT64 *,UINT32 *);
APIRET APIENTRY AcpiGetTrottling(ACPI_API_HANDLE *Hdl,UINT32 ProcId,UINT32 *Perfomance,UINT8 *Width);
APIRET APIENTRY AcpiSetTrottling(ACPI_API_HANDLE *,UINT32 ,UINT32 );
APIRET APIENTRY AcpiSetCState(ACPI_API_HANDLE *Hdl,UINT32 ProcId,UINT32 Perfomance,UINT32 State);
APIRET APIENTRY AcpiGetStatistics(ACPI_API_HANDLE *Hdl,void *Buffer);
APIRET APIENTRY AcpiSetPowerState(ACPI_API_HANDLE *Hdl,UINT32 ProcId,UINT32 State);
APIRET APIENTRY AcpiEC(ACPI_API_HANDLE *Hdl);
APIRET APIENTRY WaitDeviceNotify(ACPI_API_HANDLE *Hdl,ACPI_HANDLE *NotifyHandle,UINT32 *NotifyNumber,UINT32 What,UINT32 TimeOut);
APIRET APIENTRY WaitEmbeddedEvent(ACPI_API_HANDLE *Hdl,ACPI_HANDLE *EmbeddedHandle,UINT32 *EmbeddedNumber,UINT32 TimeOut);
unsigned char * APIENTRY AcpiStatusToStr(ACPI_STATUS Status);

#ifdef __cplusplus
}
#endif

#define WAIT_POWER_FOREVER            -1
#define SET_POWERBUTTON            0x100         // Wait Power Button
#define SET_SLEEPBUTTON            0x101         // Wait Sleep Button
#define ERROR_WRONG_VERSION        0xC0000001    // Wrong version
// For PMtimer use
#define PM_TIMER_FREQUENCY_SEC         3579545
#define PM_TIMER_FREQUENCY_MSEC        3580
#define PM_TIMER_FREQUENCY_10MSEC      35795
#define PM_TIMER_FREQUENCY_20MSEC      35795 * 2
#define PM_TIMER_FREQUENCY_MkSEC       4
#define PM_TIMER_FREQUENCY_2MkSEC      7
#define PM_TIMER_FREQUENCY_10MkSEC     36
#define PM_TIMER_FREQUENCY_20MkSEC     72
#define PM_TIMER_FREQUENCY_100MkSEC    358

#pragma pack()

#endif

