/*static char *SCCSID = "@(#)psd.h 1.0 93/18/08";*/
typedef unsigned long ulong_t;

#pragma pack(1)
// XLATOFF

#ifndef ULONG

typedef unsigned long	ULONG;
typedef unsigned short	USHORT;
typedef unsigned char	UCHAR;

#endif

#ifndef NO_ERROR
#define NO_ERROR 0
#endif

#define PSDRET int
#define PSDENTRY _stdcall

typedef PSDRET (* PSDENTRY P_F_1)(ULONG arg);
typedef PSDRET (* PSDENTRY P_F_2)(ULONG arg1, ULONG arg2);

#define PSDHelp(router, function, arg) ((*router)((function), (ULONG)(arg)))

#define WARM_REBOOT_VECTOR_SEG	0x40
#define WARM_REBOOT_VECTOR_OFF	0x67


/* PSD Info structure */

typedef struct info_s { 		  /* psd */
   ULONG  flags;			  /* PSD flags */
   ULONG  version;			  /* PSD version */
   ULONG  hmte; 			  /* MTE handle of PSD */
   UCHAR *pParmString;			  /* Pointer to ASCIIZ PSD parameter*/
   ULONG  IRQ_IPI;			  /* IRQ for IPI */
   ULONG  IRQ_LSI;			  /* IRQ for LSI */
   ULONG  IRQ_SPI;			  /* IRQ for SPI */
} PSDINFO;


/* PSD flags definition */

#define PSD_KERNEL_PIC		0x10000000  /* Let the kernel int mgr EOI d189811 */
#define PSD_ADV_INT_MODE	       0x20000000  /* PSD is in adv int mode #81531 */
#define PSD_INSTALLED		0x40000000  /* PSD has been installed */
#define PSD_INITIALIZED 	       0x80000000  /* PSD has been initalized */

/* PSD function numbers-structures */

#define PSD_INSTALL		0x00000000  /* Install PSD */

typedef struct install_s {		    /* install */
   P_F_2   pPSDHlpRouter;		    /* Address of PSDHlpRouter */
   char   *pParmString; 		    /* Pointer to parameter string */
   void   *pPSDPLMA;			    /* Pointer to PSD's PLMA */
   ULONG   sizePLMA;			    /* Size of PLMA in bytes */
} INSTALL;

#define PSD_DEINSTALL		0x00000001  /* DeInstall PSD */

#define PSD_INIT		0x00000002  /* Initialize PSD */

typedef struct init_s { 		    /* init */
   ULONG flags; 			    /* Init flags */
   ULONG version;			    /* PSD Version number */
} INIT;

#define INIT_GLOBAL_IRQ_ACCESS	0x00000001  /* Platform has global IRQ access */
#define INIT_USE_FPERR_TRAP	       0x00000002  /* Use Trap 16 to report FP err's */
#define INIT_EOI_IRQ13_ON_CPU0	0x00000004  /* eoi IRQ 13 only if on cpu 0    */
#define INIT_TIMER_CPU0 	       0x00000008  /* system timer is on CPU 0       */

/* start #89163 */
					    /* Both set and query on CPU 0    */
#define INIT_TIMER_CPU0_SET	0x00000010  /* system timer set on CPU 0      */
					    /* query OK on other processors   */
					    /* this is used for vipermp.psd   */
/* end	 #89163 */

#define PSD_PROC_INIT		0x00000003  /* Initalize processor */

#define PSD_START_PROC		0x00000004  /* Start processor */

#define PSD_GET_NUM_OF_PROCS	0x00000005  /* Get number of processors */

#define PSD_GEN_IPI		0x00000006  /* Generate an IPI */

#define PSD_END_IPI		0x00000007  /* End an IPI */

#define PSD_PORT_IO		0x00000008  /* Port I/O */

typedef struct port_io_s {		    /* port_io */
   ULONG port;				    /* Port number to access */
   ULONG data;				    /* Data read, or data to write */
   ULONG flags; 			    /* IO Flags */
} PORT_IO;

#define IO_READ_BYTE	0x0000		    /* Read a byte from the port */
#define IO_READ_WORD	0x0001		    /* Read a word from the port */
#define IO_READ_DWORD	0x0002		    /* Read a dword from the port */
#define IO_WRITE_BYTE	0x0003		    /* Write a byte to the port */
#define IO_WRITE_WORD	0x0004		    /* Write a word to the port */
#define IO_WRITE_DWORD	0x0005		    /* Write a dword to the port */

#define IO_FLAGMASK	0x0007		    /* Flag mask */

#define PSD_IRQ_MASK		0x00000009  /* Mask/Unmask IRQ levels */

typedef struct psd_irq_s {		    /* psd_irq */
   ULONG flags; 			    /* IRQ flags */
   ULONG data;				    /* IRQ data */
					    /*	 depending on type of irq */
					    /*	 operation, the data field */
					    /*	 can contain any of the */
					    /*	 following info: */
					    /*	 1) Mask or UNMasking data */
					    /*	 2) IRR or ISR reg values */
					    /*	 3) IRQ # for EOI operations */
   ULONG procnum;			    /* Processor number */
} PSD_IRQ;

#define PSD_IRQ_REG		0x0000000A  /* Access IRQ related regs */

#define PSD_IRQ_EOI		0x0000000B  /* Issue an EOI */

#define IRQ_MASK		0x00000001  /* Turn on IRQ mask bits */
#define IRQ_UNMASK		0x00000002  /* Turn off IRQ mask bits */
#define IRQ_GETMASK		0x00000004  /* Get IRQ mask bits */
#define IRQ_NEWMASK		0x00000010  /* Set and/or Reset all masks */
#define IRQ_READ_IRR		0x00000100  /* Read the IRR reg */
#define IRQ_READ_ISR		0x00000200  /* Read the ISR reg */

#define PSD_APP_COMM		0x0000000C  /* PSD/APP Communication */

#define PSD_SET_ADV_INT_MODE	0x0000000D  /* Set advanced int mode */

#define PSD_SET_PROC_STATE	0x0000000E  /* Set proc state; idle, or busy */

#define PROC_STATE_IDLE 	0x00000000  /* Processor is idle */
#define PROC_STATE_BUSY 	0x00000001  /* Processor is busy */

#define PSD_QUERY_SYSTEM_TIMER	0x0000000F  /* Query Value of System Timer 0 */

typedef struct psd_qrytmr_s {		    /* psd_qrytmr */
   ULONG qw_ulLo_psd;			    /* Timer count */
   ULONG qw_ulHi_psd;			    /* Timer count */
   ULONG pqwTmr;			    /* 16:16 ptr to qwTmr */
} PSD_QRYTMR;

#define PSD_SET_SYSTEM_TIMER	0x00000010  /* Set System Timer 0 counter    */

typedef struct psd_settmr_s {		    /* psd_settmr */
   ULONG NewRollOver;			    /* NewRollover*/
   ULONG pqwTmrRollover;		    /* 16:16 ptr to qwTmrRollover */
} PSD_SETTMR;

#define PSD_RESET_MODE		0x00000011  /* Reset Mode */
#define PSD_ICTRACE_START	0x00000012  /* Start ICTrace 148959 */

/* PSD helper function numbers-structures */

#define PSDHLP_VMALLOC		0x00000000  /* Allocate memory */

typedef struct vmalloc_s {		    /* vmalloc */
   ULONG addr;				    /* Physical address to map */
					    /*	if VMALLOC_PHYS */
					    /* Lin addr to alloc at */
					    /*	if VMALLOC_LOCSPECIFIC */
					    /* on return, addr of allocation */
   ULONG cbsize;			    /* Size of mapping in bytes */
   ULONG flags; 			    /* Allocation flags */
} VMALLOC;

#define VMALLOC_FIXED		0x00000001  /* Allocate resident memory */
#define VMALLOC_CONTIG		0x00000002  /* Allocate contigous memory */
#define VMALLOC_LOCSPECIFIC	0x00000004  /* Alloc at a specific lin address */
#define VMALLOC_PHYS		0x00000008  /* Map physical address */
#define VMALLOC_1M		0x00000010  /* Allocate below 1M */

#define VMALLOC_FLAGMASK	0x0000001f  /* Valid flag mask */

#define PSDHLP_VMFREE		0x00000001  /* Free memory */

#define PSDHLP_SET_IRQ		0x00000002  /* Set up an IRQ */

typedef struct set_irq_s {		    /* set_irq */
   USHORT irq;				    /* IRQ level */
   USHORT flags;			    /* Set IRQ flags */
   ULONG  vector;			    /* IRQ interrupt vector */
   P_F_2    handler;			    /* IRQ handler */
} SET_IRQ;

#define IRQf_IPI  0x0020		    /* IRQ for IPI */
#define IRQf_LSI  0x0040		    /* IRQ for LSI */
#define IRQf_SPI  0x0080		    /* IRQ for SPI */

#define PSDHLP_CALL_REAL_MODE	0x00000003  /* Call a function in real mode */

typedef struct call_real_mode_s {	    /* call_real_mode */
   ULONG function;			    /* Function address */
   ULONG pdata; 			    /* Pointer to data area */
} CALL_REAL_MODE;

#define PSDHLP_VMLINTOPHYS	0x00000004  /* Convert linear addr to phys */

#define PSDHLP_ADJ_PG_RANGES	0x00000005  /* Adjust page ranges */

typedef struct _pagerange_s {		    /* pagerange */
   ULONG lastframe;			    /* Last valid page in range */
   ULONG firstframe;			    /* First valid page in range */
};

typedef struct adj_pg_ranges_s{ 	    /* adj_pg_ranges */
   struct _pagerange_s *pprt;		    /* Pointer to page range table */
   ULONG nranges;			    /* Num of ranges in range table */
} ADJ_PG_RANGES;


#define  ACPI_SIZE_MEMORY_HEAP 0x10000             // 64K

// offset for IO APIC 82099AA
#define IOAP_REGSEL      0
#define IOAP_IOWIN       0x10/4
// IO APIC registers
#define IOAP_REG_ID      0                         // IO APIC ID
#define IOAP_REG_VER     1                         // IO APIC Version
#define IOAP_REG_AID     2                         // IO APIC Arbitration ID
#define IOAP_REG_RT      0x10                      // IO APIC Redirection table begin

// offset to Local APIC 82489DX (From UCHAR* offset to ULONG*)
#define LAP_ID	       0x20/4
#define LAP_VERS	       0x30/4
#define LAP_TASKPRI         0x80/4
#define LAP_EOI	       0xB0/4
#define LAP_LDEST           0xd0/4
#define LAP_DESTFMT	       0xe0/4
#define LAP_SPUR	       0xf0/4
#define LAP_ISR             0x100/4
#define LAP_IRR             0x200/4
#define LAP_ESR             0x280/4
#define LAP_ICR             0x300/4
#define LAP_ICR2            0x310/4
#define LAP_LVT_TIMER	0x320/4
#define LAP_LVT_THERMAL	0x330/4
#define LAP_LVT_PERF 	0x340/4
#define LAP_LVT_LINT0       0x350/4
#define LAP_LVT_LINT1       0x360/4
#define LAP_LVT_ERR         0x370/4

/* offsets for registers */

#define AP_IO_REG	0
#define AP_IO_DATA	4
#define AP_TASKPRI	4*8
#define AP_ID		4*2
#define AP_VERS	4*3
#define AP_LDEST	4*0xd
#define AP_DESTFMT	4*0xe
#define AP_SPUR	4*0xf
#define AP_ICMD	4*0x30
#define AP_ICMD2	4*0x31
#define AP_LVT_TIMER	4*0x32
#define AP_LVT_I0	4*0x35
#define AP_LVT_I1	4*0x36
#define AP_ICOUNT	4*0x38
#define AP_CCOUNT	4*0x39

#define AP_EOI	0xB0
#define AP_ISR	0x100
#define AP_IRR	0x200
#define APIC_ID_REG	0x20
#define AP_ESR	4*0x28
#define AP_ERR_VEC	4*0x37

/* select values for I/O registers */

#define AIR_ID	0
#define AIR_VERS	1
#define AIR_RDT	0x10
#define AIR_RDT2	0x11

/* various values for registers */

#define AV_MASK	0x10000
#define AV_TOALL	0x7fffffff
#define APIC_LOGDEST(c)	(0x40000000>>(c))   /* logical destination if ON */
#define AV_IM_OFF	0x80000000	    /* logical destination if OFF */
#define AV_FIXED	0
#define AV_LOPRI	0x100
#define AV_SMI	0x200
#define AV_NMI	0x400
#define AV_RESET	0x500
#define AV_STARTUP	0x600
#define AV_EXTINT	0x700
#define AV_PENDING	0x1000
#define AV_PDEST	0
#define AV_LDEST	0x800
#define AV_POLOW	0x2000
#define AV_POHIGH	0
#define AV_ASSERT	0x4000
#define AV_DEASSERT	0
#define AV_EDGE	0
#define AV_LEVEL	0x8000
#define AV_POLARITY	0x2000
#define AV_XTOSELF	0x40000
#define AV_XTOALL	0x80000

#define	APIC_ENABLE	0x100

/* Task priority values */
#define	TPRIHI		0xf0			
#define	TPRILO		0x00			


/* APIC error status */

#define	AP_ERR_SEND_CS	0x0001
#define	AP_ERR_RCV_CS		0x0002
#define	AP_ERR_SEND_ACPT	0x0004
#define	AP_ERR_RCV_ACPT	0x0008
#define	AP_ERR_SEND_ILLEGAL	0x0020
#define	AP_ERR_RCV_ILLEGAL	0x0040
#define	AP_ERR_ILLEGAL_REG	0x0080

#define	SPURIOUS_VEC		0xEF		/* APIC spurious vector, must be xF, any CPU (any case) set 0-3 to 1 */
#define	SPURIOUS_IRQ		46		/* APIC spurious vector */
#define	ERROR_VEC		0xF2
#define	ERROR_IRQ		47
///#define	ERROR_IRQ		0x19
#define	IPI_VEC		0xF0		/* IPI INT vector */
// #define	IPI_IRQ		0x10		/* IPI map in irqi table */
#define	IPI_IRQ		45		/* IPI map in irqi table */
#define	WARM_RESET_VEC_SEG	0x40		/* Warm reset vector segment */
#define	WARM_RESET_VEC_OFF	0x67		/* "    "      "    offset */
#define	MPAT_CONFIG_DATA	0x4AC		/* Pointer to MP config table */
#define	CPUANY			-1

/* MP+AT Interrupt Mode Control Registers */
#define IMCR_ADDR	0x22
#define IMCR_DATA	0x23

//**  RTC/CMOS equates

#define CMOS_CTRL    0x70  // RTC/CMOS control port
#define CMOS_DATA    0x71  // RTC/CMOS data port
#define CMOS_REG_B      0x8b  // CMOS register B

/* CMOS shutdown status byte */
#define	SSB		0x0f

#define MAX_IRQ               48
#define BEGIN_IRQ_VECTOR      0xA0
#define PAGE_SIZE             0x1000

#ifdef _ACPI_DEBUG
    #define SayDebug(level,prspec) \
    { printf prspec; }
#else
    #define SayDebug(level,prspec)
#endif
ULONG CLI(void);
void  STI(ULONG);

#pragma pack()
