/***************************************************************************
 *      Created 2005  eCo Software                                         *
 *                                                                         *
 *      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is        *
 *      sample code created by eCo Software. This sample code is not part  *
 *      of any standard or eCo Software product and is provided to you     *
 *      solely for the purpose of assisting you in the development of your *
 *      applications.  The code is provided "AS IS", without               *
 *      warranty of any kind. eCo Software shall not be liable for any     *
 *      damages arising out of your use of the sample code, even if they   *
 *      have been advised of the possibility of such damages.              *
 *-------------------------------------------------------------------------*/
#include <os2.h>
#include <acpi.h>
#include <acpiapi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void AcpiWalkNameSpaceTst(void);
unsigned char *AcpiStatusToStr(ACPI_STATUS);

/*
 *  Show ACPI tree with resource
 *  Pavel Shtemenko 14.08.2005
 */
ACPI_OBJECT       Object[20];

void PrintTMP(ACPI_HANDLE ObjHandle)
{
ACPI_OBJECT  *Obj;
ACPI_BUFFER  Results;
UINT32       CurTemp;
ACPI_STATUS  Status;

  Results.Length  = sizeof(Object);
  Results.Pointer = Object;

Status=AcpiEvaluateObject(ObjHandle,"_TMP",NULL,&Results);
if ( Status == AE_OK )
   {
   CurTemp = Object[0].Integer.Value;
   printf("   Current temperature: %d C ",(Object[0].Integer.Value - 2732)/10 );
   printf("( %d K * 10)\n",CurTemp );
   }
Status=AcpiEvaluateObject(ObjHandle,"_CRT",NULL,&Results);
if ( Status == AE_OK )
   {
    CurTemp = Object[0].Integer.Value;
    printf("   Critical temperature: %d C ",(Object[0].Integer.Value - 2732)/10 );
    printf("( %d K * 10)\n",CurTemp );
   }

Status=AcpiEvaluateObject(ObjHandle,"_TC1",NULL,&Results);
if ( Status == AE_OK )
   printf("   Thermal constant1: %d\n",Object[0].Integer.Value);
Status=AcpiEvaluateObject(ObjHandle,"_TC2",NULL,&Results);
if ( Status == AE_OK )
   printf("   Thermal constant2: %d\n",Object[0].Integer.Value);
Status=AcpiEvaluateObject(ObjHandle,"_TSP",NULL,&Results);
if ( Status == AE_OK )
   printf("   Thermal period: %d sec * 10\n",Object[0].Integer.Value);

}
void PrintPSL(ACPI_HANDLE ObjHandle)
{
ACPI_OBJECT  *Obj;
ACPI_BUFFER  Results;
ACPI_STATUS  Status;
UINT32       i;
ACPI_BUFFER Buffer;
char DevName[5];

  Results.Length  = sizeof(Object);
  Results.Pointer = Object;

Status=AcpiEvaluateObject(ObjHandle,"_PSL",NULL,&Results);
if (Status == AE_OK)
   {
   Obj=(ACPI_OBJECT *)Object[0].Package.Elements;
   printf("   Passive List has %d elements\n",Object[0].Package.Count);
   if (Obj[0].Reference.Handle)
      {
      Buffer.Length = 5;
      Buffer.Pointer = DevName;
      AcpiGetName(Obj[0].Reference.Handle, ACPI_SINGLE_NAME, &Buffer);
      printf("          Device [");
      for (i=0; i < 4; i++)
          printf("%c",DevName[i]);
      printf("]\n");
      }
   }
/*************************************************************************/
  Results.Length  = sizeof(Object);
  Results.Pointer = Object;

Status=AcpiEvaluateObject(ObjHandle,"_AL0",NULL,&Results);
if (Status == AE_OK)
   {
   Obj=(ACPI_OBJECT *)Object[0].Package.Elements;
   printf("   Active List has %d elements\n",Object[0].Package.Count);
   if (Obj[0].Reference.Handle)
      {
      Buffer.Length = 5;
      Buffer.Pointer = DevName;
      AcpiGetName(Obj[0].Reference.Handle, ACPI_SINGLE_NAME, &Buffer);
      printf("          Device [");
      for (i=0; i < 4; i++)
          printf("%c",DevName[i]);
      printf("]\n");
      }
   }
/*****************************************************************************/
Status=AcpiEvaluateObject(ObjHandle,"_PSV",NULL,&Results);
if ( Status == AE_OK )
   {
   i = Object[0].Integer.Value;
   printf("   Passive cooling temperature: %d C ",(Object[0].Integer.Value - 2732)/10 );
   printf("( %d K * 10 )\n",i );
   }
/*****************************************************************************/
Status=AcpiEvaluateObject(ObjHandle,"_AC0",NULL,&Results);
if ( Status == AE_OK )
   {
   i = Object[0].Integer.Value;
   printf("   Active cooling temperature: %d C ",(Object[0].Integer.Value - 2732)/10 );
   printf("( %d K * 10 )\n",i );
   }
}

/*
 * CallBack function for print object info and walk resource info
 */
ACPI_STATUS internal_PrintObjectInfo(ACPI_HANDLE ObjHandle,
				     UINT32 NestingLevel,
				     void *Context,
				     void **ReturnValue)
{
  int i;
  ACPI_BUFFER Buffer;
  ACPI_DEVICE_INFO DevInfo;
  char achName[5];
  ACPI_BUFFER       Results;
  ACPI_OBJECT       Obj,*oObj;
  ACPI_STATUS       Status;
  ULONG Lev;
  double Life;

  Results.Length  = sizeof(Object);
  Results.Pointer = &Object;


  Buffer.Length = sizeof(DevInfo);
  Buffer.Pointer = &DevInfo;
  AcpiGetObjectInfo(ObjHandle,&Buffer);
// printf("%s\n",achName);
  Buffer.Length = 5;
  Buffer.Pointer = achName;
  AcpiGetName(ObjHandle, ACPI_SINGLE_NAME, &Buffer);
  printf("Thermal zone \"%s\"\n",achName);
  PrintTMP(ObjHandle);
  PrintPSL(ObjHandle);
  

  return AE_OK;
}
/*
 * Main programm for walk ACPI tree
*/
void main(void)
{
 ACPI_STATUS       Status;
 ULONG Lev;


  printf("Show Thermal\n");
  Status=AcpiWalkNamespace(ACPI_TYPE_THERMAL, // All object types
		             ACPI_ROOT_OBJECT, // Start object
		             ACPI_UINT32_MAX, // Max depth
		             internal_PrintObjectInfo,  // Callback
                           &Lev, NULL);  // Callback param and poitner to return value
 printf("Status 0x%x %d - %s\n",Status,Status,AcpiStatusToStr(Status));
}