/***************************************************************************
 *      Created 2005  eCo Software                                         *
 *                                                                         *
 *      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is        *
 *      sample code created by eCo Software. This sample code is not part  *
 *      of any standard or eCo Software product and is provided to you     *
 *      solely for the purpose of assisting you in the development of your *
 *      applications.  The code is provided "AS IS", without               *
 *      warranty of any kind. eCo Software shall not be liable for any     *
 *      damages arising out of your use of the sample code, even if they   *
 *      have been advised of the possibility of such damages.              *
 *-------------------------------------------------------------------------*/
#include <os2.h>
#include <acpi.h>
#include <acpiapi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void AcpiWalkNameSpaceTst(void);
unsigned char *AcpiStatusToStr(ACPI_STATUS);
/*
 *  Show ACPI tree with resource
 *  Pavel Shtemenko 14.08.2005
 */
void main(void)
{
 printf("******** ACPI Tree test Programm *********\n");
 AcpiWalkNameSpaceTst();

}
/*
 * CallBack function for print resorce for _PRS ans CRS method (if possible)
 * WARNING! In any acpi table it can generate trap. For remove - need check -
 * _PRS and _CRS method would be exist.
 */
ACPI_STATUS internal_PrintResourceInfo(ACPI_RESOURCE *Resource,void *Context)
{
  UINT32 NestingLevel = (UINT32) Context;
  int i;
  for (i=0; i<NestingLevel+2; i++)
    printf(" ");
   printf(">");
  fflush(stdout);

  switch (Resource->Type)
  {
    case ACPI_RESOURCE_TYPE_IRQ:
      printf("Trigger by %s, ",Resource->Data.Irq.Triggering == ACPI_LEVEL_SENSITIVE? "Level":"Edge");
      printf("Polarity %s",Resource->Data.Irq.Polarity == ACPI_ACTIVE_LOW? "Low":"High");
      if(Resource->Data.Irq.Sharable) printf(", Sharable");
      for (i=0; i < Resource->Data.Irq.InterruptCount; i++)
	printf(" IRQ-%d", Resource->Data.Irq.Interrupts[i]);
      break;
    case ACPI_RESOURCE_TYPE_DMA:
      for (i=0; i<Resource->Data.Dma.ChannelCount; i++)
	printf(" DMA-%d", Resource->Data.Dma.Channels[i]);
      break;
    case ACPI_RESOURCE_TYPE_START_DEPENDENT:
      printf(" DPF-S: CP:0x%x PR:0x%x",Resource->Data.StartDpf.CompatibilityPriority,Resource->Data.StartDpf.PerformanceRobustness);
      break;
    case ACPI_RESOURCE_TYPE_END_DEPENDENT:
      printf(" DPF-E: CP:0x%x PR:0x%x",Resource->Data.StartDpf.CompatibilityPriority,Resource->Data.StartDpf.PerformanceRobustness);
      break;
    case ACPI_RESOURCE_TYPE_IO:
      printf(" IO:[%d 0x%x - 0x%x Al:%d Len:%d]",
	     Resource->Data.Io.IoDecode,
	     Resource->Data.Io.Minimum,
             Resource->Data.Io.Maximum,
	     Resource->Data.Io.Alignment,
	     Resource->Data.Io.AddressLength);
      break;
    case ACPI_RESOURCE_TYPE_FIXED_IO:
      printf(" FIXED-IO:[0x%x Len:%d]",
	     Resource->Data.FixedIo.Address,
	     Resource->Data.FixedIo.AddressLength);
      break;
    case ACPI_RESOURCE_TYPE_VENDOR:
      printf(" VENDOR");
      break;
    case ACPI_RESOURCE_TYPE_END_TAG:
      printf(" ENDTAG");
      break;
    case ACPI_RESOURCE_TYPE_MEMORY24:
      printf(" MEM24:[%d 0x%x - 0x%x Al:%d Len:%d]",
	     Resource->Data.Memory24.WriteProtect,
	     Resource->Data.Memory24.Minimum,
             Resource->Data.Memory24.Maximum,
	     Resource->Data.Memory24.Alignment,
	     Resource->Data.Memory24.AddressLength);
      break;
    case ACPI_RESOURCE_TYPE_MEMORY32:
      printf(" MEM32:[%d 0x%x - 0x%x Al:%d Len:%d]",
	     Resource->Data.Memory32.WriteProtect,
	     Resource->Data.Memory32.Minimum,
             Resource->Data.Memory32.Maximum,
	     Resource->Data.Memory32.Alignment,
	     Resource->Data.Memory32.AddressLength);
      break;
    case ACPI_RESOURCE_TYPE_FIXED_MEMORY32:
      printf(" FIXED-MEM32:[%d 0x%x Len:%d]",
	     Resource->Data.FixedMemory32.WriteProtect,
	     Resource->Data.FixedMemory32.Address,
	     Resource->Data.FixedMemory32.AddressLength);
      break;
    case ACPI_RESOURCE_TYPE_ADDRESS16:
      printf(" ADDRESS16:[%d 0x%x Len:%d]",
	     Resource->Data.Address16.ResourceType,
	     Resource->Data.Address16.Minimum,
	     Resource->Data.Address16.AddressLength);
      break;
    case ACPI_RESOURCE_TYPE_ADDRESS32:
      printf(" ADDRESS32:[%d 0x%x Len:%d]",
	     Resource->Data.Address32.ResourceType,
	     Resource->Data.Address32.Minimum,
	     Resource->Data.Address32.AddressLength);
      break;
    case ACPI_RESOURCE_TYPE_EXTENDED_IRQ:
      printf(" PrCm %d",Resource->Data.ExtendedIrq.ProducerConsumer);
      for (i=0; i < Resource->Data.ExtendedIrq.InterruptCount; i++)
	    printf(" IRQ%d ", Resource->Data.ExtendedIrq.Interrupts[i]);
      if ( Resource->Data.ExtendedIrq.InterruptCount )
         {
         printf("triggerred by %s, ",Resource->Data.ExtendedIrq.Triggering & ACPI_LEVEL_SENSITIVE? "Level":"Edge");
         printf("polarity %s",Resource->Data.ExtendedIrq.Polarity & ACPI_ACTIVE_LOW? "Low":"High");
         if(Resource->Data.ExtendedIrq.Sharable) printf(", Sharable");
         }
      printf(" SourceIdx %d",Resource->Data.ExtendedIrq.ResourceSource.Index);
      if( Resource->Data.ExtendedIrq.ResourceSource.StringLength)
         printf("%s",Resource->Data.ExtendedIrq.ResourceSource.StringPtr);
      break;
    case ACPI_RESOURCE_TYPE_ADDRESS64:
    default:
      printf(" (Unknown resource type: %d)", Resource->Type);
      break;
  }
  printf("\n");
  return AE_OK;
}
struct PrtLev
            {
             UINT32 CurLevel;
             UINT32 PrevLevel;
            } Lev = {0,0};
/*
 * CallBack function for print object info and walk resource info
 */
ACPI_STATUS internal_PrintObjectInfo(ACPI_HANDLE ObjHandle,
				     UINT32 NestingLevel,
				     void *Context,
				     void **ReturnValue)
{
  int i;
  ACPI_BUFFER Buffer;
  ACPI_DEVICE_INFO DevInfo;
  char achName[5];
  if(NestingLevel > 1)
  for (i=0; i < NestingLevel-1 ; i++)
    printf("   ");
  printf("|\n");
  if(NestingLevel > 1)
  for (i=0; i<NestingLevel-1 ; i++)
    printf("   ");
  printf("+*");
  memset(achName, 0, 5);
  Buffer.Length = 5;
  Buffer.Pointer = achName;
  AcpiGetName(ObjHandle, ACPI_SINGLE_NAME, &Buffer);
  printf("%s", achName);   fflush(stdout);

  Buffer.Length = sizeof(DevInfo);
  Buffer.Pointer = &DevInfo;
  AcpiGetObjectInfo(ObjHandle, &Buffer);
  printf(" T:[%s]", AcpiUtGetTypeName(DevInfo.Type));
  if (DevInfo.Valid & ACPI_VALID_HID)
    printf(" HID:[%.9s]", &(DevInfo.HardwareId));
  if (DevInfo.Valid & ACPI_VALID_CID)
    printf(" CID:[%s]", &(DevInfo.HardwareId));
  if (DevInfo.Valid & ACPI_VALID_UID)
    printf(" UID:[%.9s]", &(DevInfo.UniqueId));
  if (DevInfo.Valid & ACPI_VALID_ADR)
    printf(" ADR:[0x%x]", DevInfo.Address);
  if (DevInfo.Valid & ACPI_VALID_STA)
    printf(" STA:[0x%x]", DevInfo.CurrentStatus);
  if (DevInfo.Valid & ACPI_VALID_SXDS)
    {
    printf(" SxD State:[0x%02x:%02x:%02x:%02x]", 
             DevInfo.HighestDstates[0],
             DevInfo.HighestDstates[1],
             DevInfo.HighestDstates[2],
             DevInfo.HighestDstates[3]);
    }
  printf("\n");
  fflush(stdout);
  if (DevInfo.Type == ACPI_TYPE_DEVICE)
  {
    printf(" Current state:\n");
    AcpiWalkResources(ObjHandle,
		      METHOD_NAME__CRS,
		      internal_PrintResourceInfo,
		      (void *) NestingLevel);
    printf(" Possible State:\n");
    AcpiWalkResources(ObjHandle,
		      METHOD_NAME__PRS,
		      internal_PrintResourceInfo,
		      (void *) NestingLevel);
  }
  if (DevInfo.Type == ACPI_TYPE_THERMAL)
     {
     printf(" Thermal State:\n");
     AcpiWalkResources(ObjHandle,
		      "_TMP",
		      internal_PrintResourceInfo,
		      (void *) NestingLevel);
     }

  return AE_OK;
}
/*
 * Main programm for walk ACPI tree
*/
void AcpiWalkNameSpaceTst(void)
{
 ACPI_STATUS       Status;
  printf("Test AcpiWalkNameSpace\n");
  printf("@\n");
  Status=AcpiWalkNamespace(ACPI_TYPE_ANY, // All object types
		    ACPI_ROOT_OBJECT, // Start object
		    ACPI_UINT32_MAX, // Max depth
		    internal_PrintObjectInfo,  // Callback
                  &Lev, NULL);  // Callback param and poitner to return value
 printf("Status 0x%x %d - %s\n",Status,Status,AcpiStatusToStr(Status));
}